package com.atol.drivers.examples.paycardexample;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.atol.drivers.fptr.Fptr;
import com.atol.drivers.fptr.IFptr;
import com.atol.drivers.paycard.IPaycard;
import com.atol.drivers.paycard.Paycard;


public class PaycardExampleActivity extends Activity {
	private static final String PREFERENCES = "PREFERENCES";
	private static final String FPTR_PREF = "FPTR_PREF";
	private static final String PAYCARD_PREF = "PAYCARD_PREF";
	private static final int REQUEST_SHOW_FPTR_SETTINGS = 1;
	private static final int REQUEST_SHOW_PAYCARD_SETTINGS = 2;
	private TextView log;
	private SharedPreferences preferences;
	private LinearLayout buttonsLayout;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		preferences = getSharedPreferences(PREFERENCES, Context.MODE_PRIVATE);

		setContentView(R.layout.activity_main);
		buttonsLayout = (LinearLayout) findViewById(R.id.buttons_layout);
		Button fptrSettingsButton = (Button) findViewById(R.id.fptr_settings_button);
		Button paycardSettingsButton = (Button) findViewById(R.id.paycard_settings_button);
		Button paymentButton = (Button) findViewById(R.id.payment_button);
		Button reportButton = (Button) findViewById(R.id.report_button);
		log = (TextView) findViewById(R.id.log);

		paycardSettingsButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent intent = new Intent(PaycardExampleActivity.this, com.atol.drivers.paycard.settings.SettingsActivity.class);
				String settings = getPaycardSettings();
				if (settings == null) {
					settings = getPaycardDefaultSettings();
				}
				intent.putExtra(com.atol.drivers.paycard.settings.SettingsActivity.DEVICE_SETTINGS, settings);
				startActivityForResult(intent, REQUEST_SHOW_PAYCARD_SETTINGS);
			}
		});

		fptrSettingsButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent intent = new Intent(PaycardExampleActivity.this, com.atol.drivers.fptr.settings.SettingsActivity.class);
				String settings = getFptrSettings();
				if (settings == null) {
					settings = getFptrDefaultSettings();
				}
				intent.putExtra(com.atol.drivers.fptr.settings.SettingsActivity.DEVICE_SETTINGS, settings);
				startActivityForResult(intent, REQUEST_SHOW_FPTR_SETTINGS);
			}
		});

		paymentButton.setOnClickListener(new View.OnClickListener() {
			AsyncTask<Void, String, Void> task;

			@Override
			public void onClick(View v) {
				log.setText("");
				task = new AsyncTask<Void, String, Void>() {
					private IFptr fptr;
					private IPaycard paycard;

					private void checkError(IFptr i) throws DriverException {
						int rc = i.get_ResultCode();
						if (rc < 0) {
							String rd = i.get_ResultDescription(), bpd = null;
							if (rc == -6) {
								bpd = i.get_BadParamDescription();
							}
							if (bpd != null) {
								throw new DriverException(String.format("[%d] %s (%s)", rc, rd, bpd));
							} else {
								throw new DriverException(String.format("[%d] %s", rc, rd));
							}
						}
					}

					private void checkError(IPaycard i) throws DriverException {
						int rc = i.get_ResultCode();
						if (rc < 0) {
							String rd = i.get_ResultDescription(), bpd = null;
							if (rc == -6) {
								bpd = i.get_BadParamDescription();
							}
							if (bpd != null) {
								throw new DriverException(String.format("[%d] %s (%s)", rc, rd, bpd));
							} else {
								throw new DriverException(String.format("[%d] %s", rc, rd));
							}
						}
					}

					@Override
					protected void onPreExecute() {
						buttonsLayout.setVisibility(View.GONE);
					}

					@Override
					protected void onPostExecute(Void result) {
						buttonsLayout.setVisibility(View.VISIBLE);
					}

					@Override
					protected void onProgressUpdate(String... values) {
						if (values == null || values.length == 0) {
							return;
						}
						logAdd(values[0]);
					}

					@Override
					protected Void doInBackground(Void... params) {
						fptr = new Fptr();
						paycard = new Paycard();

						try {
							fptr.create(getApplication());
							paycard.create(getApplication());

							publishProgress("Загрузка настроек...");
							if (fptr.put_DeviceSettings(getFptrSettings()) < 0) {
								checkError(fptr);
							}
							if (paycard.put_DeviceSettings(getPaycardSettings()) < 0) {
								checkError(paycard);
							}
							paycard.put_PinPadDevice(fptr.get_PinPadDevice());
							paycard.put_ModemDevice(fptr.get_ModemDevice());

							publishProgress("Установка соединения...");
							if (fptr.put_DeviceEnabled(true) < 0) {
								checkError(fptr);
							}
							if (paycard.put_DeviceEnabled(true) < 0) {
								checkError(paycard);
							}
							publishProgress("OK");

							publishProgress("Проверка связи с ККМ...");
							if (fptr.GetStatus() < 0) {
								checkError(fptr);
							}
							publishProgress("OK");

							publishProgress("Продажа...");
							if (paycard.put_AuthorizationType(IPaycard.AUTHORIZATION_READER_PIN) < 0) {
								checkError(paycard);
							}
							if (paycard.put_OperationType(IPaycard.OPERATION_SUB) < 0) {
								checkError(paycard);
							}
							if (paycard.put_Sum(10.0) < 0) {
								checkError(paycard);
							}
							if (paycard.put_CharLineLength(fptr.get_CharLineLength()) < 0) {
								checkError(paycard);
							}
							if (paycard.PrepareAuthorization() < 0) {
								checkError(paycard);
							}
							if (paycard.Authorization() < 0) {
								checkError(paycard);
							}
							publishProgress("OK");

							publishProgress("Печать слипа...");
							if (fptr.put_Caption(paycard.get_Text()) < 0) {
								checkError(fptr);
							}
							if (fptr.put_TextWrap(IFptr.WRAP_LINE) < 0) {
								checkError(fptr);
							}
							if (fptr.put_Alignment(IFptr.ALIGNMENT_LEFT) < 0) {
								checkError(fptr);
							}
							if (fptr.PrintString() < 0) {
								checkError(fptr);
							}
							if (fptr.put_Mode(IFptr.MODE_REPORT_NO_CLEAR) < 0) {
								checkError(fptr);
							}
							if (fptr.SetMode() < 0) {
								checkError(fptr);
							}
							if (fptr.PrintFooter() < 0) {
								checkError(fptr);
							}
							publishProgress("OK");

						} catch (Exception e) {
							publishProgress(e.toString());
						} finally {
							fptr.destroy();
							paycard.destroy();
						}
						return null;
					}
				};
				task.execute();
			}
		});

		reportButton.setOnClickListener(new View.OnClickListener() {
			AsyncTask<Void, String, Void> task;

			@Override
			public void onClick(View v) {
				log.setText("");
				task = new AsyncTask<Void, String, Void>() {
					private IFptr fptr;
					private IPaycard paycard;

					private void checkError(IFptr i) throws DriverException {
						int rc = i.get_ResultCode();
						if (rc < 0) {
							String rd = i.get_ResultDescription(), bpd = null;
							if (rc == -6) {
								bpd = i.get_BadParamDescription();
							}
							if (bpd != null) {
								throw new DriverException(String.format("[%d] %s (%s)", rc, rd, bpd));
							} else {
								throw new DriverException(String.format("[%d] %s", rc, rd));
							}
						}
					}

					private void checkError(IPaycard i) throws DriverException {
						int rc = i.get_ResultCode();
						if (rc < 0) {
							String rd = i.get_ResultDescription(), bpd = null;
							if (rc == -6) {
								bpd = i.get_BadParamDescription();
							}
							if (bpd != null) {
								throw new DriverException(String.format("[%d] %s (%s)", rc, rd, bpd));
							} else {
								throw new DriverException(String.format("[%d] %s", rc, rd));
							}
						}
					}

					@Override
					protected void onPreExecute() {
						buttonsLayout.setVisibility(View.GONE);
					}

					@Override
					protected void onPostExecute(Void result) {
						buttonsLayout.setVisibility(View.VISIBLE);
					}

					@Override
					protected void onProgressUpdate(String... values) {
						if (values == null || values.length == 0) {
							return;
						}
						logAdd(values[0]);
					}

					@Override
					protected Void doInBackground(Void... params) {
						fptr = new Fptr();
						paycard = new Paycard();

						try {
							fptr.create(getApplication());
							paycard.create(getApplication());

							publishProgress("Загрузка настроек...");
							if (fptr.put_DeviceSettings(getFptrSettings()) < 0) {
								checkError(fptr);
							}
							if (paycard.put_DeviceSettings(getPaycardSettings()) < 0) {
								checkError(paycard);
							}
							paycard.put_PinPadDevice(fptr.get_PinPadDevice());
							paycard.put_ModemDevice(fptr.get_ModemDevice());

							publishProgress("Установка соединения...");
							if (fptr.put_DeviceEnabled(true) < 0) {
								checkError(fptr);
							}
							if (paycard.put_DeviceEnabled(true) < 0) {
								checkError(paycard);
							}
							publishProgress("OK");

							publishProgress("Проверка связи с ККМ...");
							if (fptr.GetStatus() < 0) {
								checkError(fptr);
							}
							publishProgress("OK");

							publishProgress("Сверка итогов...");
							if (paycard.put_ReportType(IPaycard.REPORT_CLOSE_SESSION) < 0) {
								checkError(paycard);
							}
							if (paycard.put_CharLineLength(fptr.get_CharLineLength()) < 0) {
								checkError(paycard);
							}
							if (paycard.BeginReport() < 0) {
								checkError(paycard);
							}
							if (paycard.EndReport() < 0) {
								checkError(paycard);
							}
							publishProgress("OK");

							publishProgress("Печать слипа...");
							if (fptr.put_Caption(paycard.get_Text()) < 0) {
								checkError(fptr);
							}
							if (fptr.put_TextWrap(IFptr.WRAP_LINE) < 0) {
								checkError(fptr);
							}
							if (fptr.put_Alignment(IFptr.ALIGNMENT_LEFT) < 0) {
								checkError(fptr);
							}
							if (fptr.PrintString() < 0) {
								checkError(fptr);
							}
							if (fptr.put_Mode(IFptr.MODE_REPORT_NO_CLEAR) < 0) {
								checkError(fptr);
							}
							if (fptr.SetMode() < 0) {
								checkError(fptr);
							}
							if (fptr.PrintFooter() < 0) {
								checkError(fptr);
							}
							publishProgress("OK");

						} catch (Exception e) {
							publishProgress(e.toString());
						} finally {
							fptr.destroy();
							paycard.destroy();
						}
						return null;
					}
				};
				task.execute();
			}
		});
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		switch (requestCode) {
		case REQUEST_SHOW_FPTR_SETTINGS:
			if (data != null && data.getExtras() != null) {
				String settings = data.getExtras().getString(com.atol.drivers.fptr.settings.SettingsActivity.DEVICE_SETTINGS);
				setFptrSettings(settings);
			}
			break;
		case REQUEST_SHOW_PAYCARD_SETTINGS:
			if (data != null && data.getExtras() != null) {
				String settings = data.getExtras().getString(com.atol.drivers.paycard.settings.SettingsActivity.DEVICE_SETTINGS);
				setPaycardSettings(settings);
			}
			break;
		}
	}

	private String getFptrSettings() {
		return preferences.getString(FPTR_PREF, getFptrDefaultSettings());
	}

	private void setFptrSettings(String settings) {
		SharedPreferences.Editor editor = preferences.edit();
		editor.putString(FPTR_PREF, settings);
		editor.apply();
	}

	private String getFptrDefaultSettings() {
		IFptr fprint = new Fptr();
		fprint.create(this);
		String settings = fprint.get_DeviceSettings();
		fprint.destroy();
		return settings;
	}

	private String getPaycardSettings() {
		return preferences.getString(PAYCARD_PREF, getPaycardDefaultSettings());
	}

	private void setPaycardSettings(String settings) {
		SharedPreferences.Editor editor = preferences.edit();
		editor.putString(PAYCARD_PREF, settings);
		editor.apply();
	}

	private String getPaycardDefaultSettings() {
		IPaycard paycard = new Paycard();
		paycard.create(this);
		String settings = paycard.get_DeviceSettings();
		paycard.destroy();
		return settings;
	}

	private void logAdd(String string) {
		String value = (String) log.getText();
		if (value.length() > 2096) {
			value = value.substring(0, 2096);
		}
		log.setText(string + "\n" + value);
	}

	private static class DriverException extends Exception {
		public DriverException(String msg) {
			super(msg);
		}
	}
}
